#include "PacmanInterface2D.h"
#include <Color.h>
#include <2D/Hardware2D.h>
#include <Log/LogPtr.h>
#include <String.hpp>
#include <Converter.h>
#include <safe_new.h>

Interface2D * create_game(Hardware2D * hardware)
{
	return new PacmanInterface2D( hardware );
}

PacmanInterface2D::PacmanInterface2D(Hardware2D * hardware)
:	Interface2D( hardware ),
	scene( hardware ),
	fps_text( &scene, 3, "data\\small_alphabet" ),
	game( &scene ),
	time( 0 ),
	fps( 2000 ),
	need_quit( false )
{
	hardware->set_mode( 240, 240 );

	fps_text.move( 0, 0 );
	fps_text.change_text( "Unknown FPS" );
}

PacmanInterface2D::~PacmanInterface2D()
{
	hardware->stop_mode();
}

bool PacmanInterface2D::render()
{
	bool ok = scene.render( 0, 0, Color(0,0,128) );
	return ok;
}

bool PacmanInterface2D::life_cycle(float delta_time)
{
	fps.record( time );
	LogPtr()->message( String("Fps ") + Converter::convert( fps.get_fps() ) + "\n" );

	fps_text.change_text( String("Fps ") + Converter::convert( int(fps.get_fps()) ) );

	time += delta_time;
	game.life_cycle( delta_time );
	return !need_quit;
}

void PacmanInterface2D::key_down(int scan_code)
{
	game.key_down( scan_code );
}

void PacmanInterface2D::key_up(int scan_code)
{
	game.key_up( scan_code );
}
